import os

SAMPLE_DATA_PATH = os.path.realpath(
    os.path.join(
        os.path.realpath(__file__),
        "..",
        "..",
        "..",
        "..",
        "sample_data",
        "sample.json",
    )
)
